﻿using System.Collections.Generic;

namespace Model
{
    public class MapSegmentBank
    {
        public List<int[,]> corners;
        public List<int[,]> sides;
        public List<int[,]> internals;

        public MapSegmentBank()
        {
            corners = new List<int[,]>();
            sides = new List<int[,]>();
            internals = new List<int[,]>();

            PopulateCorners();
            PopulateSides();
            PopulateInternals();
        }

        public void PopulateCorners()
        {
            corners.Add(new int[10, 10]
            {
                { 2, 2, 2, 2, 2, 2, 2, 2, 2, 2 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 1, 1, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 1, 1, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 1, 1, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 1, 1, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
            }
            );
            
            corners.Add(new int[10, 10]
            {
                { 2, 2, 2, 2, 2, 2, 2, 2, 2, 2 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 1, 1, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 1, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 1, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 1, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 1, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
            }
            );
            corners.Add(new int[10, 10]
            {
                { 2, 2, 2, 2, 2, 2, 2, 2, 2, 2 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 1, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 1, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 1, 0, 0 },
                { 2, 0, 0, 0, 1, 1, 1, 1, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
            }
            );

        }
        
        public void PopulateSides()
        {
            sides.Add(new int[10, 10]
            {
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 1, 0, 0, 1, 0, 0, 0 },
                { 2, 0, 0, 1, 1, 1, 1, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 1, 1, 1, 1, 0, 0, 0 },
                { 2, 0, 0, 1, 0, 0, 1, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
            }
            );
            sides.Add(new int[10, 10]
            {
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 1, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 1, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 1, 0 },
                { 2, 0, 1, 1, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 1, 1, 0, 0, 0, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 1, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 1, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 1, 0, 0, 0 },
                { 2, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
            }
            );
            
        }

        public void PopulateInternals()
        {
            internals.Add(new int[10, 10]
            {
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 1, 1, 0, 0, 0, 0, 1, 1, 0 },
                { 0, 1, 1, 0, 0, 0, 0, 1, 1, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 1, 1, 0, 0, 0, 0, 1, 1, 0 },
                { 0, 1, 1, 0, 0, 0, 0, 1, 1, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
            }
            );
            internals.Add(new int[10, 10]
            {
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 1, 1, 0, 0, 0, 0, 1, 1, 0 },
                { 0, 1, 1, 0, 0, 0, 0, 1, 1, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 1, 1, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 1, 1, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 1, 1, 0, 0, 0, 0, 1, 1, 0 },
                { 0, 1, 1, 0, 0, 0, 0, 1, 1, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
            }
            );
            internals.Add(new int[10, 10]
            {
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 1, 0, 0, 1, 1, 1, 0 },
                { 0, 0, 0, 1, 0, 0, 1, 0, 0, 0 },
                { 0, 1, 1, 1, 0, 0, 1, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 1, 1, 1, 0, 0, 0, 0, 0, 0 },
                { 0, 1, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 1, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
            }
            );
            internals.Add(new int[10, 10]
            {
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 1, 0, 0, 1, 0, 0, 0 },
                { 0, 0, 0, 1, 0, 0, 1, 0, 0, 0 },
                { 0, 1, 1, 1, 0, 0, 1, 1, 1, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 1, 1, 1, 0, 0, 1, 1, 1, 0 },
                { 0, 0, 0, 1, 0, 0, 1, 0, 0, 0 },
                { 0, 0, 0, 1, 0, 0, 1, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
            }
            );
            internals.Add(new int[10, 10]
            {
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 1, 0, 0, 0, 0, 1, 0, 0 },
                { 0, 0, 0, 1, 0, 0, 1, 0, 0, 0 },
                { 0, 0, 0, 0, 1, 1, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 1, 1, 0, 0, 0, 0 },
                { 0, 0, 0, 1, 0, 0, 1, 0, 0, 0 },
                { 0, 0, 1, 0, 0, 0, 0, 1, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
            }
            );
            
        }
        /* TEMPLATE
            corners.Add(new int[10, 10]
            {
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
                { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }
            }
            );
            */
    }
}