﻿using NetworkBehaviourClasses;
using UnityEngine;

namespace Model
{
    public class TaskValidator
    {
        private GameObject _playerObject;

        public TaskValidator(GameObject playerObject)
        {
            _playerObject = playerObject;
        }

        public bool TryValidate()
        {
            Vector3Int playerPos = Vector3Int.RoundToInt(_playerObject.transform.position);
            int mapHalf = MapManager.Instance.mapSize * 10 / 2;
            GameObject moveObject = _playerObject.GetComponent<Movable>().moveObject;

            bool playerPosOk = playerPos.x == -mapHalf || playerPos.x == mapHalf - 1 || playerPos.y == -mapHalf ||
                               playerPos.y == mapHalf - 1;
            if (!playerPosOk) return false;

            bool cubesPosOk = true;
            for (int i = 0; i < moveObject.transform.childCount; i++)
            {
                GameObject child = moveObject.transform.GetChild(i).gameObject;
                if (!child.CompareTag("Cube")) continue;

                Vector3Int childPos = Vector3Int.RoundToInt(child.transform.position);
                if (childPos.x >= -mapHalf && childPos.x <= mapHalf - 1 && childPos.y >= -mapHalf &&
                    childPos.y <= mapHalf - 1)
                {
                    cubesPosOk = false;
                    break;
                }
            }

            if (!cubesPosOk) return false;

            int points = TaskManager.Instance.currentTask.points;
            Team team = _playerObject.GetComponent<Player>().team;
            
            bool valid = TaskManager.Instance.ValidatePattern(moveObject);
            if (valid)
            {
                GameManager.Instance.AddPoints(team, points);
                MapManager.Instance.ScatterMoveObject(moveObject);
                TaskManager.Instance.CreateNewTask();
            }

            return valid;
        }
    }
}