using System;
using System.Threading.Tasks;
using TMPro;
using Unity.Services.Authentication;
using Unity.Services.Core;
using UnityEngine;
using UnityEngine.Serialization;

namespace MultiPlayer
{
    public class Authentication : MonoBehaviour
    {
        public static Authentication Instance { get; private set; }
        
        void Awake()
        {
            if (Instance != null && Instance != this)
            {
                Destroy(gameObject);
                return;
            }
            Instance = this;
        }

        /// <summary>
        /// Used to initialize Unity Services and Authenticate Players
        /// </summary>
        /// <param name="playerNameParam">Name to Authenticate as</param>
        public async Task Authenticate(string playerNameParam)
        {

            InitializationOptions initializationOptions = new InitializationOptions();
            initializationOptions.SetProfile(playerNameParam);
            await UnityServices.InitializeAsync(initializationOptions);
            AuthenticationService.Instance.SignedIn += () =>
            {
                Debug.Log($"<color=green>Signed In: {AuthenticationService.Instance.PlayerId},{playerNameParam}</color>");
            };

            await AuthenticationService.Instance.SignInAnonymouslyAsync();
        }
    }
}
