﻿using System;
using System.Threading.Tasks;
using Unity.Netcode;
using Unity.Netcode.Transports.UTP;
using Unity.Networking.Transport.Relay;
using Unity.Services.Relay;
using Unity.Services.Relay.Models;
using UnityEngine;

namespace MultiPlayer.Relay
{
    public class RelayManager : MonoBehaviour
    {
        public static RelayManager Instance { get; private set; }
        
        
        
        void Awake()
        {
            if (Instance != null && Instance != this)
            {
                Destroy(gameObject);
                return;
            }

            Instance = this;
            DontDestroyOnLoad(this);
        }

        public async Task<string> CreateRelay(int maxPlayers)
        {
            try
            {
                Allocation allocation = await RelayService.Instance.CreateAllocationAsync(maxPlayers - 1);
                string joinCode = await RelayService.Instance.GetJoinCodeAsync(allocation.AllocationId);
                Debug.Log($"<color=green>RelayCreated:{joinCode}</color>");

                RelayServerData relayServerData = new RelayServerData(allocation, "dtls");
                
                NetworkManager.Singleton.GetComponent<UnityTransport>().SetRelayServerData(relayServerData);

                NetworkManager.Singleton.StartHost();
                return joinCode;
            }
            catch (Exception e)
            {
                Debug.LogException(e);
                return null;
            }
        }

        
        
        public async void JoinRelay(string joinCode)
        {
            try
            {
                JoinAllocation joinAllocation = await RelayService.Instance.JoinAllocationAsync(joinCode);
                
                RelayServerData relayServerData = new RelayServerData(joinAllocation, "dtls");
                
                NetworkManager.Singleton.GetComponent<UnityTransport>().SetRelayServerData(relayServerData);

                NetworkManager.Singleton.StartClient();
                
                Debug.Log($"<color=green>JoinedRelay{joinCode}</color>");
            }
            catch (Exception e)
            {
                Debug.LogException(e);
            }
        }
    }
}