using System;
using System.Collections.Generic;
using Action;
using Model;
using UI.LobbyScripts;
using Unity.Netcode;
using UnityEngine;
using UnityEngine.Serialization;

namespace NetworkBehaviourClasses
{
    public class GameManager : NetworkBehaviour
    {
        // game settings
        [SerializeField] public float turnLength = 5; // turn time in seconds
        [SerializeField] private int turns = 100; // number of turns before game ends
    
        // private fields
        private GameState _state; // game state
    
        // public fields
        public static GameManager Instance { get; private set; } // singleton
        public static event Action<GameState> OnGameStateChanged;
        public static event Action<Team, int> OnPointsAdded;
        public NetworkVariable<float> TurnTimer { get; private set; } = new NetworkVariable<float>();
        public int RemainingTurns;
        public Dictionary<Team, int> points;

        // private methods
        private void Awake()
        {
            if (Instance != null && Instance != this)
            {
                Destroy(gameObject);
                return;
            }

            RemainingTurns = turns;
            Instance = this;
            ActionValidator.OnActionsValidated += ActionValidatorOnActionsValidated;
        }

        private void ActionValidatorOnActionsValidated(List<RobotAction> actions)
        {
            foreach (var action in actions)
            {
                action.doAction();
            }
            UpdateGameState(GameState.Turn);
        }

        public override void OnNetworkSpawn()
        {
            if(!IsHost) return;
            TurnTimer.Value = 0;
            _state = GameState.Turn;
            
            points = new Dictionary<Team, int>
            {
                [Team.Red] = 0,
                [Team.Blue] = 0
            };

            base.OnNetworkSpawn();
        }

        private void Update()
        {
            if(!IsHost) return;

            TimerTick();
        }

        private void TimerTick()
        {
            
            if (_state == GameState.Turn)
            {
                TurnTimer.Value += Time.deltaTime;
            
                if (TurnTimer.Value >= turnLength)
                {
                    TurnTimer.Value = 0;
                    RemainingTurns--;
                    
                    UpdateGameState(GameState.Move); 
                    //UpdateGameState(GameState.Turn);
                }
                if (RemainingTurns == 0)
                {
                    UpdateGameState(GameState.GameOver);
                }
            }
        }
    
        // public methods
        public void UpdateGameState(GameState newState)
        {
            _state = newState;
            OnGameStateChanged?.Invoke(newState);
        }

        public void AddPoints(Team team, int p)
        {
            points[team] += p;
            OnPointsAdded?.Invoke(team, p);
        }

    }

    public enum GameState
    {
        Turn,
        Move,
        GameOver
    }
    public enum Team
    {
        Red,
        Blue
    };

}