﻿using System;
using JetBrains.Annotations;
using Model;
using MultiPlayer;
using UI.DevConsole;
using UI.InputSystem;
using Unity.Netcode;
using UnityEngine;

namespace NetworkBehaviourClasses
{
    public class InputSystem : NetworkBehaviour
    {
        private Vector3[] _worldPos;
        private bool _isSelected;
        private bool _twoInputSelect;
        private bool _activeSelection;
        public static event Action<EventArgs> OnMoveOver;

        public override void OnNetworkSpawn()
        {
            if(!IsOwner) { return;}
            _worldPos = new Vector3[2];
            
            InputUI.Instance.SelectedChanged += ShowPattern;
            GameManager.OnGameStateChanged += GameManagerOnGameStateChanged;
            OnMoveOver += InputSystemOnMoveOver;
        }

        private void InputSystemOnMoveOver(EventArgs e)
        {
            if(!IsOwner) {return;}
            SendData();
        }

        private void GameManagerOnGameStateChanged(GameState state)
        {
            if (!IsOwner) {return;}
            if (state == GameState.Move)
            {
                MoveOverClientRpc();
            }
        }

        [ClientRpc]
        private void MoveOverClientRpc()
        {
            OnMoveOver?.Invoke(EventArgs.Empty);
            ShowPossibleInputs.Instance.ResetPattern();
            _twoInputSelect = false;
        }
        

        private void Update()
        {
            if(!IsOwner) {return;}

            if (Input.GetMouseButtonDown(0))
            {
                if (!_twoInputSelect)
                {
                    PossibleInput(Camera.main.ScreenToWorldPoint(Input.mousePosition));
                }
                else
                {
                    PossibleInputTwoSelect(Camera.main.ScreenToWorldPoint(Input.mousePosition));
                }
            }

            if (Input.GetKeyDown(KeyCode.N))
            {
                SendData();
            }
            
        }
        private void ShowPattern(object sender, EventArgs eventArgs)
        {

            if (InputUI.Instance.Selected.type is ActionType.Move or ActionType.Connect or ActionType.Disconnect or ActionType.Clean)
            {
                ShowPossibleInputs.Instance.ShowCrossPattern(transform.position,false);            
                _twoInputSelect = false;
            }
            else if(InputUI.Instance.Selected.type == ActionType.Rotate)
            {
                ShowPossibleInputs.Instance.ShowCrossWithTurns(transform.position);
                _twoInputSelect = false;
            }
            else if (InputUI.Instance.Selected.type == ActionType.Wait)
            { 
                ShowPossibleInputs.Instance.ResetPattern();
                _twoInputSelect = false;
            }
            else if (InputUI.Instance.Selected.type is ActionType.Link or ActionType.Unlink )
            {
                ShowPossibleInputs.Instance.ResetPattern();
                _twoInputSelect = true;
                _activeSelection = false;
            }
        }

        private void PossibleInput(Vector3 input)
        {

            if(!ShowPossibleInputs.Instance.Selected(input)) return;

            _isSelected = ShowPossibleInputs.Instance.IsCurrentlySelected();

            
            _worldPos[0] = input;
            _worldPos[1] = new Vector3(0, 0, 0);
            //Debug.Log($"Selected Tile {_worldPos[0]}");
        }

        private void PossibleInputTwoSelect(Vector3 input)
        {

            if(!ShowPossibleInputs.Instance.Selected(input)) _activeSelection = false;
            
            if (!_activeSelection)
            {
                ShowPossibleInputs.Instance.ShowCrossPattern(input,true);
                _worldPos[0] = input;
                _activeSelection = true;
                return;
            }
            
            _isSelected = ShowPossibleInputs.Instance.IsCurrentlySelected();
            _worldPos[1] = input;
        }

        [ClientRpc]
        private void SendDataClientRpc()
        {
        }
        
        private void SendData()
        { 
            //Debug.Log($"Sending Data as {_isSelected}");
            if (_isSelected)
            {
                GameNetworkManager.Instance.SendDataServerRpc(GetComponent<NetworkObject>(), _worldPos,InputUI.Instance.Selected.type);
                //Debug.Log($"Sent Input{_worldPos[0]},{_worldPos[1]}");
            }
            else
            {
                GameNetworkManager.Instance.SendDataServerRpc(GetComponent<NetworkObject>(), _worldPos, ActionType.Wait);
                //Debug.Log($"Sent Input{_worldPos[0]},{_worldPos[1]}");
            }
        }
    }
    
}
