using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.Rendering;
using UnityEngine.Tilemaps;

namespace NetworkBehaviourClasses
{
    public class VisibilityManager : MonoBehaviour
    {
        // Private fields
        [SerializeField] 
        private Tilemap Overlay;

        // Two fields for possible other colorings
        [SerializeField] 
        private float maxAppearance;

        [SerializeField] 
        private Color MinAppearanceColor, MaxAppearanceColor, NoAppearanceColor;
        
        // Collection of the currently appearing tiles
        private Dictionary<Vector3Int, float> AppearingTiles = new Dictionary<Vector3Int, float>();
        private Dictionary<Vector3Int, float> DisappearingTiles = new Dictionary<Vector3Int, float>();

        // Visibility utilities
        [SerializeField]
        public float testAddVisibilityAmount;

        [SerializeField]
        private float appearanceFallOff;

        // Manhattan distance radius
        [SerializeField] 
        public int testRadius;

        // Singleton Contructor
        public static VisibilityManager Instance { get; private set; }
        void Start()
        {
            if (Instance != null && Instance != this)
            {
                Destroy(gameObject);
            }
            Instance = this;
        }
        
        // Public methods
        public void AddVisibility(Vector2 worldPosition, float appearanceAmount, int radius)
        {
            
            Vector3Int gridPosition = Overlay.WorldToCell(worldPosition);

            for (int x = -radius; x<=radius; x++)
            {
                for (int y = -radius; y <= radius; y++)
                {
                    float distanceFromCenter = Math.Abs(x) + Math.Abs(y);
                    if (distanceFromCenter <= radius)
                    {
                        
                        Vector3Int nextTilePosition =
                            new Vector3Int(gridPosition.x + x, gridPosition.y + y, 0);
                        DisappearingTiles.Add(nextTilePosition,-1f);
                        ChangeVisibility(nextTilePosition,1);
                        
                    }
                }
            }
            
            ChangeVisibility(gridPosition,appearanceAmount);
            ChangeFullVisible();
            VisualizeAppearance();
            DisappearingTiles.Clear();
            
        }
        // Change visibility for the semi-visible tiles
        private void ChangeVisibility(Vector3Int gridPosition, float changeBy)
        {
            
            if (!AppearingTiles.ContainsKey(gridPosition))
            {
                AppearingTiles.Add(gridPosition,0f);

                float newValue = changeBy;
                
                AppearingTiles[gridPosition] = changeBy - 0.4f;

            }
            
        }
        // change visibility for the fully visible tiles
        private void ChangeFullVisible()
        {
            foreach (var x in DisappearingTiles)
            {
                ChangeVisibility(x.Key,-1f);
            }
        }

        private void VisualizeAppearance()
        {
            foreach (var entry in AppearingTiles)
            {
                float AppearancePercent = entry.Value ;
                Color newTileColor = Color.black *AppearancePercent;
                
                Overlay.SetTileFlags(entry.Key, TileFlags.None);
                Overlay.SetColor(entry.Key,newTileColor);
                Overlay.SetTileFlags(entry.Key, TileFlags.LockColor);
            }

            foreach (var entry in DisappearingTiles)
            {
                float AppearancePercent = entry.Value ;
                Color newTileColor = Color.black *AppearancePercent;
                
                Overlay.SetTileFlags(entry.Key, TileFlags.None);
                Overlay.SetColor(entry.Key,newTileColor);
                Overlay.SetTileFlags(entry.Key, TileFlags.LockColor);
            }
        }

        // Update event function
       /* 
        private void Update()
        {
            if (Input.GetMouseButtonDown(0))
            {
                Vector2 mousePosition = Camera.main.ScreenToWorldPoint(Input.mousePosition);
                
                AddVisibility(mousePosition, testAddVisibilityAmount, testRadius);

            }
        }
        */

    }
}