﻿using System;
using System.Collections.Generic;
using System.Reflection;
using Unity.VisualScripting;

namespace UI.DevConsole
{
    public class CommandSetup
    {
        private readonly Assembly[] _assemblies = AppDomain.CurrentDomain.GetAssemblies();
        private readonly Dictionary<KeyValuePair<Type,MethodInfo>, DevCommands> _commandsToMethods;

        public CommandSetup()
        {
            
            _commandsToMethods = new Dictionary<KeyValuePair<Type,MethodInfo>, DevCommands>();

            foreach (var assembly in _assemblies)
            {
                if (assembly.GetName().Name != "Assembly-CSharp") continue;
                var types = assembly.GetTypes();
                foreach (var type in types)
                {
                    var members = type.GetMethods(BindingFlags.NonPublic | BindingFlags.Public | BindingFlags.Instance);
                    foreach (var member in members)
                    {
                        var command = member.GetCustomAttribute(typeof(DevCommands),false) as DevCommands;
                        if (command == null) continue;
                        if (command.Name == null)
                        {
                            command.Name = member.Name;
                        }
                        _commandsToMethods.Add(new KeyValuePair<Type, MethodInfo>(type,member), command);
                    }
                }
            }
        }

        public Dictionary<KeyValuePair<Type,MethodInfo>, DevCommands> GetCommandDictionary()
        {
            return _commandsToMethods;
        }
    }
}