﻿using System;
using System.Collections.Generic;
using System.Reflection;
using TMPro;
using Unity.VisualScripting;
using UnityEngine;
using UnityEngine.UI;

namespace UI.DevConsole
{
    public class CommandsRunner : MonoBehaviour
    {
        [SerializeField]
        private string prefix;

        private CommandSetup _commandSetup;
        private DevConsole _devConsole;
        private string _inputValue;

        private CommandsRunner _instance;
        
        [SerializeField]
        private int size=22;
        
        
        [Header("UI")] 
        [SerializeField] private GameObject devConsoleUI;
        [SerializeField] private TMP_InputField input;
        [SerializeField] private GameObject debugWin;
        [SerializeField] private KeyCode LogOpen;
        
        private DevConsole DevConsole
        {
            get
            {
                if (_devConsole != null){return _devConsole;}
                return _devConsole = new DevConsole(prefix,CommandSetup);
            }
        }

        private CommandSetup CommandSetup
        {
            get
            {
                if (_commandSetup != null) { return _commandSetup;}
                return _commandSetup = new CommandSetup();
            }
        }
        
        public void Awake()
        {
            if (_instance != null && _instance != this)
            {
                Destroy(gameObject);
                return;
            }

            _instance = this;
            
            DontDestroyOnLoad(gameObject);

            debugWin.GetComponent<Text>().text = "";
            
            DevConsole.RunCommand += EventRunCommand;
            DevConsole.FailedCommand += FailedCommand;
            DevConsole.SucessCommand += SucessCommand;
        }

        void OnEnable()
        {
            Application.logMessageReceived += HandelLog;
        }

        public void Update()
        {
            if (!Input.GetKeyDown(LogOpen))
            {
                return;
            }
            
            if (devConsoleUI.activeSelf)
            {
                devConsoleUI.SetActive(false);
            }
            else
            {
                devConsoleUI.SetActive(true);
                input.ActivateInputField();
            }

        }

        private void RunCommand(KeyValuePair<KeyValuePair<Type, MethodInfo>, DevCommands> commands,object[] args)
        {
            var runObject = FindObjectOfType(commands.Key.Key);
            var component = runObject.GetComponent(commands.Key.Key);
            commands.Key.Value.Invoke(component,args);
        }
        private void EventRunCommand(object sender, RunCommandEventArgs e)
        {
            RunCommand(e.Command,e.Args);
            
        }

        private void SucessCommand(object sender, EventArgs e)
        {
            debugWin.GetComponent<Text>().text += $"<color=Cyan> > {_inputValue}</color>\n";
            size += 36;
            debugWin.GetComponent<RectTransform>().SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical,size);
        }
        
        private void FailedCommand(object sender, EventArgs e)
        {
            debugWin.GetComponent<Text>().text += $"<color=Red> > {_inputValue}</color>\n";
            size += 36;
            debugWin.GetComponent<RectTransform>().SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical,size);
        }

        public void ProcessCommand(string inputValue)
        {
            _inputValue = inputValue;
            try
            {
                DevConsole.ProcessInput(inputValue);
            }
            catch (Exception e)
            {
                Debug.LogWarning(e.Message);
            }
            input.text = string.Empty;
        }
        
        private void HandelLog(string logString, string stackTrace, LogType type)
        {

            switch (type)
            {
                case LogType.Warning:
                case LogType.Assert:
                    debugWin.GetComponent<Text>().text += $"<color=yellow>[{DateTime.Now.ToLongTimeString()}] {logString}</color>\n";
                    break;
                case LogType.Exception:
                case LogType.Error:
                    debugWin.GetComponent<Text>().text += $"<color=red>[{DateTime.Now.ToLongTimeString()}] {logString}</color>\n";
                    break;
                case LogType.Log:
                    debugWin.GetComponent<Text>().text += $"[{DateTime.Now.ToLongTimeString()}] {logString}\n";
                    break;
            }
            
            size += 36;
            debugWin.GetComponent<RectTransform>().SetSizeWithCurrentAnchors(RectTransform.Axis.Vertical,size);
        }
        
        [DevCommands]
        private void Log(string logText)
        {
            Debug.Log(logText);
        }
    }
}