﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using UnityEngine;

namespace UI.DevConsole
{
    public class DevConsole
    {
        private readonly CommandSetup _commandSetup;
        private readonly string _prefix;
        private bool _foundCommand;
        public event EventHandler<RunCommandEventArgs> RunCommand;
        public event EventHandler FailedCommand; 
        public event EventHandler SucessCommand; 

        public DevConsole(string prefix, CommandSetup commandSetup )
        {
            _prefix = prefix;
            _commandSetup = commandSetup;
            _foundCommand = false;
        }

        public void ProcessInput(string inputValue)
        {
            _foundCommand = false;
            if(!inputValue.StartsWith(_prefix)) return;

            inputValue = inputValue.Remove(0, _prefix.Length);

            string[] inputSplit = inputValue.Split(' ');
            string commandInput = inputSplit[0];
            string[] args = inputSplit.Skip(1).ToArray();
            List<object> argsfin = new List<object>();
            uint count = 0;

            foreach (var command in _commandSetup.GetCommandDictionary())
            {
                if (!string.Equals(command.Value.Name, commandInput,StringComparison.OrdinalIgnoreCase)) continue;

                var parameters = command.Key.Value.GetParameters();
                
                foreach (var parameter in parameters)
                {   
                    try
                    {
                        Type type = parameter.ParameterType;
                        argsfin.Add(TypeDescriptor.GetConverter(type).ConvertFromString(args[count]));
                        count++;

                    }
                    catch (Exception e)
                    {
                        throw(e);
                    }
                }
                
                _foundCommand = true;
                SucessCommand?.Invoke(this, EventArgs.Empty);
                RunCommand?.Invoke(this, new RunCommandEventArgs(command, argsfin.ToArray()));
            }

            if (_foundCommand == false)
            {
                FailedCommand?.Invoke(this, EventArgs.Empty);
            }
        }
        
        
        
    }
}