using System;
using UnityEngine;
using UnityEngine.Tilemaps;

namespace UI.InputSystem
{
    public class InputUI : MonoBehaviour
    {

        
        [Header("InputButtons")]
        [SerializeField] private InputButtonUI[] buttons;
        [SerializeField] private Color baseColor;

        [Header("ShowInputs")] 
        [SerializeField]  private Tilemap overlay;
        public InputButtonUI Selected { get; private set; }

        public event EventHandler SelectedChanged; 
    
        public static InputUI Instance { get; private set; }
        private void Awake()
        {
            
            if (Instance != null && Instance != this)
            {
                Destroy(gameObject);
            }
            Instance = this;
            
            foreach (InputButtonUI button in buttons)
            {
                button.baseColor = baseColor;
                button.SelectedEvent += NewSelection;
            }
        }

        private void NewSelection(object sender, EventArgs e)
        {
            if (Selected != (InputButtonUI)sender)
            {
                Selected = (InputButtonUI)sender;
                foreach (InputButtonUI button in buttons)
                {
                    if (button != (InputButtonUI)sender)
                    {
                        button.IsSelected = false;
                    }
                }
                SelectedChanged?.Invoke(this,EventArgs.Empty);
            }
        }
    }
}
