﻿using System;
using System.Collections.Generic;
using Unity.Netcode;
using UnityEngine;
using UnityEngine.Serialization;
using UnityEngine.Tilemaps;

namespace UI.InputSystem
{
    public class ShowPossibleInputs : MonoBehaviour
    {
        [SerializeField] private Tilemap overlayMap;

        [SerializeField] private Color appearanceColor;
        [SerializeField] private Color hideColor;
        [Header("Normal")]
        [SerializeField] private Tile basicWhite;
        [SerializeField] private Tile turnRight;
        [SerializeField] private Tile turnLeft;
        [Header("Selected")]
        [FormerlySerializedAs("SelectedTile")] [SerializeField] private Tile selectedTile;
        [SerializeField] private Tile selectedTileRight;
        [SerializeField] private Tile selectedTileLeft;

        public static ShowPossibleInputs Instance { get; private set; }

        private List<Vector3Int> _currentCross;
        private Vector3Int _selectedPos;

        private bool _normalCross;
        
        private void Awake()
        {
            if (Instance != null && Instance != this)
            {
                Destroy(gameObject);
            }
            Instance = this;
            _currentCross = new List<Vector3Int>();
        }

        public  void ShowCrossPattern(Vector3 center , bool cubeSelection)
        {
             ResetPattern();

            _currentCross.Add(overlayMap.WorldToCell(center));
            _currentCross.Add(_currentCross[0]+new Vector3Int(1, 0, 0));
            _currentCross.Add(_currentCross[0]+new Vector3Int(-1, 0, 0));
            _currentCross.Add(_currentCross[0]+new Vector3Int(0, 1, 0));
            _currentCross.Add(_currentCross[0]+new Vector3Int(0, -1, 0));
            
            foreach (var gridPosition in _currentCross)
            {
            
                overlayMap.SetTileFlags(gridPosition, TileFlags.None);
                overlayMap.SetColor(gridPosition,hideColor);
                overlayMap.SetTileFlags(gridPosition, TileFlags.LockColor);
            }

            if (cubeSelection)
            {
                overlayMap.SetTileFlags(_currentCross[0], TileFlags.None);
                overlayMap.SetTile(_currentCross[0], selectedTile);
                overlayMap.SetColor(_currentCross[0],Color.white);
                overlayMap.SetTileFlags(_currentCross[0], TileFlags.LockColor);
            }
            
            _selectedPos = _currentCross[0];
            _normalCross = true;
            overlayMap.gameObject.SetActive(true);
        }

        public void ShowCrossWithTurns(Vector3 center)
        {
            ResetPattern();

            _currentCross.Add(overlayMap.WorldToCell(center));
            _currentCross.Add(_currentCross[0]+new Vector3Int(1, 0, 0));
            _currentCross.Add(_currentCross[0]+new Vector3Int(-1, 0, 0));
            
            overlayMap.SetTileFlags(_currentCross[2], TileFlags.None);
            overlayMap.SetTile(_currentCross[2],turnLeft);
            overlayMap.SetColor(_currentCross[2],Color.white);
            overlayMap.SetTileFlags(_currentCross[2], TileFlags.LockColor);
            
            
            overlayMap.SetTileFlags(_currentCross[1], TileFlags.None);
            overlayMap.SetTile(_currentCross[1],turnRight);
            overlayMap.SetColor(_currentCross[1],Color.white);
            overlayMap.SetTileFlags(_currentCross[1], TileFlags.LockColor);
            
            overlayMap.SetTileFlags(_currentCross[0], TileFlags.None);
            overlayMap.SetColor(_currentCross[0],hideColor);
            overlayMap.SetTileFlags(_currentCross[0], TileFlags.LockColor);
            
            
            _selectedPos = _currentCross[0];
            _normalCross = false;
            overlayMap.gameObject.SetActive(true);
        }

        public  bool Selected(Vector3 selected)
        {
            var input = overlayMap.WorldToCell(selected);
            
            if (_currentCross.Contains(input) && _currentCross[0] != input)
            {
                if (_normalCross)
                {
                    foreach (var pos in _currentCross)
                    {
                        if (pos == input)
                        {
                            overlayMap.SetTileFlags(pos, TileFlags.None);
                            overlayMap.SetTile(pos, selectedTile);
                            overlayMap.SetColor(pos,Color.white);
                            overlayMap.SetTileFlags(pos, TileFlags.LockColor);
                        }
                    }

                    if (_selectedPos != _currentCross[0])
                    {
                        overlayMap.SetTile(_selectedPos, basicWhite);
                        overlayMap.SetTileFlags(_selectedPos, TileFlags.None);
                        overlayMap.SetColor(_selectedPos, hideColor);
                        overlayMap.SetTileFlags(_selectedPos, TileFlags.LockColor);
                    }
                    if(_selectedPos == input){_selectedPos = _currentCross[0];}
                    else
                    {
                        _selectedPos = input;
                    }
                    
                }
                else
                {
                    if (input == _currentCross[1])
                    {
                        overlayMap.SetTileFlags(input, TileFlags.None);
                        overlayMap.SetTile(input, selectedTileRight);
                        overlayMap.SetColor(input, Color.white);
                        overlayMap.SetTileFlags(input, TileFlags.LockColor);
                    }
                    else if(input == _currentCross[2])
                    {
                        overlayMap.SetTileFlags(input, TileFlags.None);
                        overlayMap.SetTile(input, selectedTileLeft);
                        overlayMap.SetColor(input, Color.white);
                        overlayMap.SetTileFlags(input, TileFlags.LockColor);
                    }

                    if (_selectedPos == _currentCross[1] && _selectedPos != _currentCross[0])
                    {
                        
                        overlayMap.SetTile(_selectedPos,turnRight);
                        overlayMap.SetTileFlags(_selectedPos, TileFlags.None);
                        overlayMap.SetColor(_selectedPos, Color.white);
                        overlayMap.SetTileFlags(_selectedPos, TileFlags.LockColor);
                        if(_selectedPos == input){_selectedPos = _currentCross[0];}
                        else
                        {
                            _selectedPos = input;
                        }
                    }
                    else if(_selectedPos == _currentCross[2] && _selectedPos != _currentCross[0])
                    {
                        
                        overlayMap.SetTile(_selectedPos,turnLeft);
                        overlayMap.SetTileFlags(_selectedPos, TileFlags.None);
                        overlayMap.SetColor(_selectedPos, Color.white);
                        overlayMap.SetTileFlags(_selectedPos, TileFlags.LockColor);
                        if(_selectedPos == input){_selectedPos = _currentCross[0];}
                        else
                        {
                            _selectedPos = input;
                        }
                        
                    }
                    else
                    {
                        _selectedPos = input;
                    }
                    
                }

                return true;
            }
            else
            {
                return false;
            }
        }
        
        public void ResetPattern()
        {
            foreach (var gridPosition in _currentCross)
            {
                overlayMap.SetTile(gridPosition,basicWhite);
                overlayMap.SetTileFlags(gridPosition, TileFlags.None);
                overlayMap.SetColor(gridPosition,appearanceColor);
                overlayMap.SetTileFlags(gridPosition, TileFlags.LockColor);
            }
            _currentCross.Clear();
            overlayMap.gameObject.SetActive(false);
        }

        public bool IsCurrentlySelected()
        {
            if (_selectedPos != _currentCross[0]) return true;
            return false;
        }
        
    }
}