using MultiPlayer.LobbyScripts;
using TMPro;
using Unity.Services.Authentication;
using Unity.Services.Lobbies.Models;
using UnityEngine;
using UnityEngine.Serialization;
using UnityEngine.UI;

namespace UI.LobbyScripts
{
    public class JoinedLobbyUI : MonoBehaviour
    {
        [FormerlySerializedAs("BackButton")] 
        [SerializeField] private Button backButton;

        [SerializeField] private Button startGame;
    
        [Header("Prefabs")] 
    
        [FormerlySerializedAs("RedPrefab")]
        [SerializeField] private Transform redPrefab;
        [FormerlySerializedAs("BluePrefab")] 
        [SerializeField] private Transform bluePrefab;

        [Header("Containers")]
    
        [FormerlySerializedAs("RedContainer")]
        [SerializeField] private Transform redContainer;
        [FormerlySerializedAs("BlueContainer")] 
        [SerializeField] private Transform blueContainer;

        [Header("UI")] 
    
        [FormerlySerializedAs("LobbyName")]
        [SerializeField] private TextMeshProUGUI lobbyName;
        [FormerlySerializedAs("LobbyNumber")] 
        [SerializeField] private TextMeshProUGUI lobbyNumber;

        public Lobby Lobby { get; private set; }

        public static JoinedLobbyUI Instance { get; private set; }
    
        void Awake()
        {
            if (Instance != null && Instance != this)
            {
                Destroy(gameObject);
                return;
            }

            Instance = this;

        }

        public void Start()
        {
            LobbyManager.Instance.LobbyChangedEvent += (sender, args) =>
            {
                SetUpUI(args.changedLobby);
            };
        
            backButton.onClick.AddListener(() =>
            {
                LobbyManager.Instance.LeaveLobby();
                LobbyUI.Instance.gameObject.SetActive(true);
                gameObject.SetActive(false);
            });
            startGame.onClick.AddListener(() =>
            {
                LobbyManager.Instance.StartGame();
            });
            startGame.gameObject.SetActive(false);
        }


        /// <summary>
        /// Displays name and max players on UI
        /// </summary>
        /// <param name="lobby">The lobby to display</param>
        public void SetUpUI(Lobby lobby)
        {
            lobbyName.text = lobby.Name;
            lobbyNumber.text = $"{lobby.MaxPlayers-lobby.AvailableSlots}/{lobby.MaxPlayers}";
            UpdatePlayers(lobby);
        }

        
        /// <summary>
        /// Updates player displays based on the lobby
        /// </summary>
        /// <param name="lobby">Lobby to display from</param>
        private void UpdatePlayers(Lobby lobby)
        {
            foreach (Transform child in redContainer)
            {
                if(child == redPrefab) continue;
            
                Destroy(child.gameObject);
            }
        
        
            foreach (Transform child in blueContainer)
            {
                if(child == bluePrefab) continue;
            
                Destroy(child.gameObject);
            }
        
            this.Lobby = lobby;
            foreach (Player player in lobby.Players)
            {
                if (player.Data["Team"].Value == "Red")
                {
                    Transform redPlayer = Instantiate(redPrefab, redContainer);
                    JoinedPlayerUI script = redPlayer.GetComponent<JoinedPlayerUI>();
                    script.SetUpPlayerStat(player);

                    if (player.Id != AuthenticationService.Instance.PlayerId)
                    {
                        script.Switch.gameObject.SetActive(false);
                    }
                }
                if (player.Data["Team"].Value == "Blue")
                {
                    Transform bluePlayer = Instantiate(bluePrefab, blueContainer);
                    JoinedPlayerUI script = bluePlayer.GetComponent<JoinedPlayerUI>();
                    script.SetUpPlayerStat(player);
                    if (player.Id != AuthenticationService.Instance.PlayerId)
                    {
                        script.Switch.gameObject.SetActive(false);
                    }
                }

                if (AuthenticationService.Instance.PlayerId == lobby.HostId)
                {
                    startGame.gameObject.SetActive(true);
                }
            }
        }
    }
}
