using System;
using System.Collections.Generic;
using MultiPlayer.LobbyScripts;
using TMPro;
using Unity.Services.Lobbies.Models;
using UnityEngine;
using UnityEngine.Serialization;
using UnityEngine.UI;

namespace UI.LobbyScripts
{
    public class LobbyUI : MonoBehaviour
    {
        public static LobbyUI Instance { get; private set; }
        
        [Header("Parents")]
        [SerializeField] private Transform canvas;
        [SerializeField] private Transform container;
        
        [Header("PreFabs")]
        [FormerlySerializedAs("NewLobbyTemplate")] 
        [SerializeField] private Transform newLobbyTemplate;
        [SerializeField] private Transform listTemplate;

        [Header("Buttons")] 
        [SerializeField] private Button addLobbyButton;
        [SerializeField] private Button refreshLobby;

        [FormerlySerializedAs("JoinedLobby")] 
        [SerializeField] private GameObject joinedLobby;

        private float _refreshTimer = 2f;

        private void Awake()
        {
            if (Instance != null && Instance != this)
            {
                Destroy(gameObject);
                return;
            }

            Instance = this;
        }

        private void Start()
        {
            
            addLobbyButton.onClick.AddListener(OnNewLobbyCreate);
            refreshLobby.onClick.AddListener(OnRefresh);
            LobbyManager.Instance.LobbyListChangedEvent += (sender, args) =>
            {
                UpdateLobbyList(args.lobbies);
            };
        }

        private void Update()
        {
            RefreshTimer();
        }


        /// <summary>
        /// Called when the new lobby button is clicked
        /// </summary>
        private void OnNewLobbyCreate()
        {
            Transform newLobby = Instantiate(newLobbyTemplate, canvas);
            newLobby.Find("Ok").GetComponent<Button>().onClick.AddListener(() =>
            {
                string lobbyName = newLobby.gameObject.GetComponent<NewLobbyUI>().lobbyName.GetComponentInChildren<TextMeshProUGUI>().text;
                int maxPlayers = Convert.ToInt32(newLobby.gameObject.GetComponent<NewLobbyUI>().lobbyMaxPlayer
                    .GetComponentInChildren<TextMeshProUGUI>().text);
                LobbyManager.Instance.CreateLobby(lobbyName,maxPlayers);
                gameObject.SetActive(false);
                JoinedLobbyUI.Instance.gameObject.SetActive(true);
                Destroy(newLobby.gameObject);
            });
            newLobby.Find("Cancle").GetComponent<Button>().onClick.AddListener(() =>
            {
                Destroy(newLobby.gameObject);
            });
        }

        /// <summary>
        /// Refresh Timer so we cant overwhelm requests
        /// </summary>
        private void RefreshTimer()
        {
            if (_refreshTimer > 0)
            {
                _refreshTimer -= Time.deltaTime;
            }
        }
        
        /// <summary>
        /// Function to call when Refresh button is pressed
        /// </summary>
        private void OnRefresh()
        {
            if (!(_refreshTimer < 0)) return;
            LobbyManager.Instance.ListLobbys();
            _refreshTimer = 2f;
        }

        /// <summary>
        /// Updates the current list of lobbies
        /// </summary>
        /// <param name="lobbies">Lobbies to display</param>
        private void UpdateLobbyList(List<Lobby> lobbies)
        {
            foreach (Transform child in container)
            {
                if(child == listTemplate) continue;
                
                Destroy(child.gameObject);
            }

            foreach (Lobby lobby in lobbies)
            {
                Transform lobbyList = Instantiate(listTemplate, container);
                LobbyListUI lobbyListUI = lobbyList.GetComponent<LobbyListUI>();
                lobbyListUI.UpdateLobby(lobby,canvas);
            }
        }
    }
}
