using System;
using System.Linq;
using TMPro;
using UnityEngine;
using UnityEngine.Serialization;
using UnityEngine.UI;

namespace UI.LobbyScripts
{
    public class NewLobbyUI : MonoBehaviour
    {
        public Button Ok;
        public Button Cancle;

        [FormerlySerializedAs("LobbyName")] 
        [SerializeField] public Button lobbyName;
        [FormerlySerializedAs("LobbyMaxPlayer")] 
        [SerializeField] public Button lobbyMaxPlayer;
        [FormerlySerializedAs("InputPrefabObject")] 
        [SerializeField] private GameObject inputPrefabObject;
        private GameObject _inputObject;
    
        void Start()
        {
            lobbyName.onClick.AddListener(LobbyNameInput);
            lobbyMaxPlayer.onClick.AddListener(LobbyMaxPlayerInput);
            Cancle.onClick.AddListener(() =>
            {
                Destroy(gameObject);
            });
        }

        /// <summary>
        /// Makes input form and gets data from it when the name is pressed
        /// </summary>
        private void LobbyNameInput()
        {
            if(_inputObject != null ) {return;}
            _inputObject = Instantiate(inputPrefabObject, transform.parent);
            InputPrefab script = _inputObject.GetComponent<InputPrefab>();
            script.SetUpText("Name Lobby","Enter Lobby Name");
        
            script.Ok.onClick.AddListener(() =>
            {
                if (!String.IsNullOrEmpty(script.Input.text))
                {
                    lobbyName.GetComponentInChildren<TextMeshProUGUI>().text = script.Input.text;
                    Destroy(_inputObject);
                }
                else
                {
                    script.Input.text = "<color=red>Invalid Input</color>";
                }
            });
            
            script.Input.onEndEdit.AddListener((_) =>
            {
                if (!String.IsNullOrEmpty(script.Input.text))
                {
                    lobbyName.GetComponentInChildren<TextMeshProUGUI>().text = script.Input.text;
                    Destroy(_inputObject);
                }
                else
                {
                    script.Input.text = "<color=red>Invalid Input</color>";
                }
            });
        
            script.Cancle.onClick.AddListener(() =>
            {
                Destroy(_inputObject);
            });
        }

        /// <summary>
        /// Makes input form and gets data from it when the max player field is pressed
        /// </summary>
        private void LobbyMaxPlayerInput()
        {
            if(_inputObject != null ) {return;}
            _inputObject = Instantiate(inputPrefabObject, transform.parent);
            InputPrefab script = _inputObject.GetComponent<InputPrefab>();
            script.SetUpText("Max Player Number","Enter Player Max");
        
            script.Ok.onClick.AddListener(() =>
            {
                if (script.Input.text.All(char.IsDigit) && !String.IsNullOrEmpty(script.Input.text))
                {
                    lobbyMaxPlayer.GetComponentInChildren<TextMeshProUGUI>().text = script.Input.text;
                    Destroy(_inputObject);
                }
                else
                {
                    script.Input.text = "<color=red>Invalid Input</color>";
                }

            });
            
            script.Input.onEndEdit.AddListener((_) =>
            {
                if (script.Input.text.All(char.IsDigit) && !String.IsNullOrEmpty(script.Input.text))
                {
                    lobbyMaxPlayer.GetComponentInChildren<TextMeshProUGUI>().text = script.Input.text;
                    Destroy(_inputObject);
                }
                else
                {
                    script.Input.text = "<color=red>Invalid Input</color>";
                }
            });
        
            script.Cancle.onClick.AddListener(() =>
            {
                Destroy(_inputObject);
            });
        }
    }
}
