using System.Globalization;
using NetworkBehaviourClasses;
using TMPro;
using Unity.Netcode;
using UnityEngine;

namespace UI
{
    public class TurnTimerUI : NetworkBehaviour
    {
        // serialized fields
        [SerializeField] private TextMeshProUGUI counterText;
        [SerializeField] private TextMeshProUGUI timerText;
        
        // private fields
        private CultureInfo _culture;

        // private methods
        private void Start()
        {
            _culture = new CultureInfo("en-US");
            UpdateCounter(GameManager.Instance.RemainingTurns);
            GameManager.OnGameStateChanged += GameManagerOnOnGameStateChanged;
        }

        private void GameManagerOnOnGameStateChanged(GameState state)
        {
            if (state == GameState.Turn)
            {
                UpdateCounterClientRpc(GameManager.Instance.RemainingTurns);
            }
        }

        [ClientRpc]
        private void UpdateCounterClientRpc(int val)
        {
            UpdateCounter(val);
        }

        private void Update()
        {
            UpdateTimer();
        }
        
        private void UpdateCounter(int count)
        {
            counterText.text = count.ToString();
        }

        private void UpdateTimer()
        {
            timerText.text = (GameManager.Instance.turnLength - GameManager.Instance.TurnTimer.Value).ToString("N2", _culture);
        }
    }
}